import 'dart:math';

import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:info_tren/components/cupertino_divider.dart';
import 'package:info_tren/components/sliver_persistent_header_padding.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/pages/train_info_page/train_info_constants.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_cupertino_DisplayTrainStation.dart';
import 'package:info_tren/utils/state_to_string.dart';

class TrainInfoLoadingCupertino extends TrainInfoLoadingShared {
  TrainInfoLoadingCupertino({required super.title, super.loadingText, super.key,});

  @override
  Widget build(BuildContext context) {
    return CupertinoPageScaffold(
      navigationBar: CupertinoNavigationBar(
        middle: Text(title),
      ),
      child: Center(
        child: loadingWidget,
      ),
    );
  }
}

class TrainInfoErrorCupertino extends TrainInfoErrorShared {
  const TrainInfoErrorCupertino({
    required super.error,
    required super.title,
    super.refresh,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return CupertinoPageScaffold(
      navigationBar: CupertinoNavigationBar(
        middle: Text(title),
      ),
      child: Center(
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Text(error.toString()),
            if (refresh != null)
              Padding(
                padding: const EdgeInsets.all(8),
                child: CupertinoButton(
                  child: const Text('Retry'),
                  onPressed: () => refresh!(),
                ),
              ),
          ],
        ),
      ),
    );
  }
}

class TrainInfoCupertino extends TrainInfoShared {
  const TrainInfoCupertino({
    required super.trainData,
    super.refresh,
    super.isRefreshing,
    super.onViewYesterdayTrain,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return CupertinoPageScaffold(
      navigationBar: CupertinoNavigationBar(
        middle: Text("Informații despre ${trainData.rank} ${trainData.number}"),
        trailing: refresh == null ? null : isRefreshing == true ? const CupertinoActivityIndicator() : CupertinoButton(
          padding: const EdgeInsets.all(0),
          alignment: Alignment.center,
          child: const Icon(CupertinoIcons.refresh),
          onPressed: () => refresh!(),
        ),
      ),
      child: SafeArea(
        top: false,
        bottom: false,
        child: TrainInfoBody(
          trainData: trainData,
          onViewYesterdayTrain: onViewYesterdayTrain,
        ),
      ),
    );

    // return CupertinoPageScaffold(
    //   navigationBar: CupertinoNavigationBar(
    //     middle: Text(title ?? ""),
    //   ),
    //   child: SafeArea(
    //     bottom: false,
    //     child: FutureBuilder<OnDemandTrainData>(
    //       future: TrainDataWebViewAdapter.of(context).trainData(onInvalidation: () {
    //         Navigator.of(context).pop();
    //       }),
    //       builder: (context, snapshot) {
    //         if (!snapshot.hasData) {
    //           return Center(
    //             child: CupertinoActivityIndicator(),
    //           );
    //         }

    //         try {
    //           Future.wait([
    //             snapshot.data.rang,
    //             snapshot.data.trainNumber
    //           ]).then((values) {
    //             setState(() {
    //               title = "Informații despre ${values[0]} ${values[1]}";
    //             });
    //           });

    //           return CustomScrollView(
    //             slivers: <Widget>[
    //               DisplayTrainID(data: snapshot.data,),
    //               DisplayTrainOperator(data: snapshot.data,),
    //               DisplayTrainRoute(data: snapshot.data,),
    //               DisplayTrainDeparture(data: snapshot.data,),
    //               SliverToBoxAdapter(
    //                 child: CupertinoDivider(
    //                   color: FOREGROUND_WHITE,
    //                 ),
    //               ),
    //               DisplayTrainLastInfo(data: snapshot.data,),
    //               SliverToBoxAdapter(
    //                 child: CupertinoDivider(),
    //               ),
    //               SliverToBoxAdapter(
    //                 child: IntrinsicHeight(
    //                   child: Row(
    //                     children: <Widget>[
    //                       Expanded(
    //                         child: DisplayTrainNextStop(data: snapshot.data,),
    //                       ),
    //                       SizedBox(
    //                         height: double.infinity,
    //                         child: CupertinoVerticalDivider(),
    //                       ),
    //                       Expanded(
    //                         child: DisplayTrainDestination(data: snapshot.data,),
    //                       )
    //                     ],
    //                   ),
    //                 ),
    //               ),
    //               SliverToBoxAdapter(
    //                 child: CupertinoDivider(),
    //               ),
    //               SliverToBoxAdapter(
    //                 child: IntrinsicHeight(
    //                   child: Row(
    //                     children: <Widget>[
    //                       Expanded(
    //                         child: DisplayTrainRouteDuration(data: snapshot.data,),
    //                       ),
    //                       SizedBox(
    //                         height: double.infinity,
    //                         child: CupertinoVerticalDivider(),
    //                       ),
    //                       Expanded(
    //                         child: DisplayTrainRouteDistance(data: snapshot.data,),
    //                       )
    //                     ],
    //                   ),
    //                 ),
    //               ),
    //               SliverToBoxAdapter(
    //                 child: CupertinoDivider(
    //                   color: FOREGROUND_WHITE,
    //                 ),
    //               ),
    //               DisplayTrainStations(
    //                 data: snapshot.data,
    //                 pageLoadFuture: TrainDataWebViewAdapter.of(context).nextLoadFuture,
    //               ),
    //             ],
    //           );
    //         }
    //         on OnDemandInvalidatedException {
    //           Navigator.of(context).pop();
    //           print("Got OnDemandInvalidatedException!");
    //           return Container();
    //         }
    //       },
    //     ),
    //   ),
    // );
  }
}

class TrainInfoBodyCupertino extends TrainInfoBodyShared {
  const TrainInfoBodyCupertino({
    super.key,
    required super.trainData,
    super.onViewYesterdayTrain,
    super.isRefreshing,
    super.refresh,
  });

  @override
  Widget build(BuildContext context) {
    final mq = MediaQuery.of(context);
    final topPadding = mq.padding.top;

    if (mq.orientation == Orientation.landscape && mq.size.width >= 1000) {
      return Row(
        mainAxisSize: MainAxisSize.max,
        children: [
          Container(
            constraints: const BoxConstraints(
              minWidth: 400,
              maxWidth: 400,
            ),
            child: Column(
              mainAxisSize: MainAxisSize.max,
              children: [
                DisplayTrainID(trainData: trainData),
                DisplayTrainOperator(trainData: trainData),
                DisplayTrainRoute(trainData: trainData),
                DisplayTrainDeparture(trainData: trainData),
                const CupertinoDivider(
                  color: foregroundWhite,
                ),
                DisplayTrainLastInfo(trainData: trainData),
                const CupertinoDivider(),
                IntrinsicHeight(
                  child: Row(
                    children: <Widget>[
                      // Expanded(
                      //   child: DisplayTrainNextStop(trainData: trainData,),
                      // ),
                      Expanded(
                        child: DisplayTrainRouteDuration(
                          trainData: trainData,
                        ),
                      ),
                      // Expanded(
                      //   child: DisplayTrainDestination(trainData: trainData,),
                      // ),
                      const SizedBox(
                        height: double.infinity,
                        child: CupertinoVerticalDivider(),
                      ),
                      Expanded(
                        child: DisplayTrainRouteDistance(
                          trainData: trainData,
                        ),
                      ),
                    ],
                  ),
                ),
                const CupertinoDivider(
                  color: foregroundWhite,
                ),
                if (onViewYesterdayTrain != null && trainData.stations.first.departure!.scheduleTime.compareTo(DateTime.now()) > 0) ...[
                  DisplayTrainYesterdayWarningCupertino(onViewYesterdayTrain!),
                  const CupertinoDivider(
                    color: foregroundWhite,
                  ),
                ],
              ],
            ),
          ),
          Expanded(
            child: NestedScrollView(
              headerSliverBuilder: (context, innerBoxIsScrolled) {
                return [
                  // SliverPadding(
                  //   padding: EdgeInsets.only(
                  //     top: topPadding,
                  //   ),
                  // ),
                  SliverPersistentHeaderPadding(
                    maxHeight: topPadding,
                  )
                ];
              },
              body: CustomScrollView(
                slivers: [
                  if (refresh != null)
                    CupertinoSliverRefreshControl(
                      builder: (context, mode, pulledExtent,
                          refreshTriggerPullDistance, refreshIndicatorExtent) {
                        return Column(
                          mainAxisSize: MainAxisSize.min,
                          children: [
                            SizedBox(
                              height: pulledExtent,
                              child: Column(
                                children: [
                                  SizedBox(
                                    height: min(
                                        refreshIndicatorExtent, pulledExtent),
                                    child: Center(
                                      child: Builder(
                                        builder: (context) {
                                          if (mode ==
                                              RefreshIndicatorMode.inactive) {
                                            return Container();
                                          } else if (mode ==
                                              RefreshIndicatorMode.done) {
                                            return const Text('Refreshed!');
                                          } else if (mode ==
                                              RefreshIndicatorMode.drag) {
                                            return Row(
                                              mainAxisSize: MainAxisSize.min,
                                              children: const [
                                                CupertinoActivityIndicator(
                                                  animating: false,
                                                ),
                                                Text('Pull to refresh...'),
                                              ],
                                            );
                                          } else if (mode ==
                                              RefreshIndicatorMode.armed) {
                                            return Row(
                                              mainAxisSize: MainAxisSize.min,
                                              children: const [
                                                CupertinoActivityIndicator(
                                                  animating: false,
                                                ),
                                                Text('Release to refresh...'),
                                              ],
                                            );
                                          } else {
                                            return Row(
                                              mainAxisSize: MainAxisSize.min,
                                              children: const [
                                                CupertinoActivityIndicator(),
                                                Text('Refreshing'),
                                              ],
                                            );
                                          }
                                        },
                                      ),
                                    ),
                                  ),
                                  Expanded(
                                    child: Container(),
                                  ),
                                ],
                              ),
                            ),
                          ],
                        );
                      },
                      onRefresh: refresh,
                    ),

                  DisplayTrainStations(
                    trainData: trainData,
                  ),
                  SliverToBoxAdapter(
                    child: Container(
                      height: MediaQuery.of(context).viewPadding.bottom,
                    ),
                  ),
                ],
              ),
            ),
          ),
        ],
      );
    }

    return NestedScrollView(
      headerSliverBuilder: (context, innerBoxIsScrolled) {
        return [
          // SliverPadding(
          //   padding: EdgeInsets.only(
          //     top: topPadding,
          //   ),
          // ),
          SliverPersistentHeaderPadding(
            maxHeight: topPadding,
          )
        ];
      },
      body: Builder(builder: (context) {
        return CustomScrollView(
          slivers: <Widget>[
            if (refresh != null)
              CupertinoSliverRefreshControl(
                builder: (context, mode, pulledExtent,
                    refreshTriggerPullDistance, refreshIndicatorExtent) {
                  return Column(
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      SizedBox(
                        height: pulledExtent,
                        child: Column(
                          children: [
                            SizedBox(
                              height: min(
                                  refreshIndicatorExtent, pulledExtent),
                              child: Center(
                                child: Builder(
                                  builder: (context) {
                                    if (mode ==
                                        RefreshIndicatorMode.inactive) {
                                      return Container();
                                    } else if (mode ==
                                        RefreshIndicatorMode.done) {
                                      return const Text('Refreshed!');
                                    } else if (mode ==
                                        RefreshIndicatorMode.drag) {
                                      return Row(
                                        mainAxisSize: MainAxisSize.min,
                                        children: const [
                                          CupertinoActivityIndicator(
                                            animating: false,
                                          ),
                                          Text('Pull to refresh...'),
                                        ],
                                      );
                                    } else if (mode ==
                                        RefreshIndicatorMode.armed) {
                                      return Row(
                                        mainAxisSize: MainAxisSize.min,
                                        children: const [
                                          CupertinoActivityIndicator(
                                            animating: false,
                                          ),
                                          Text('Release to refresh...'),
                                        ],
                                      );
                                    } else {
                                      return Row(
                                        mainAxisSize: MainAxisSize.min,
                                        children: const [
                                          CupertinoActivityIndicator(),
                                          Text('Refreshing'),
                                        ],
                                      );
                                    }
                                  },
                                ),
                              ),
                            ),
                            Expanded(
                              child: Container(),
                            ),
                          ],
                        ),
                      ),
                    ],
                  );
                },
                onRefresh: refresh,
              ),
            ...[
              DisplayTrainID(
                trainData: trainData,
              ),
              DisplayTrainOperator(
                trainData: trainData,
              ),
              DisplayTrainRoute(
                trainData: trainData,
              ),
              DisplayTrainDeparture(
                trainData: trainData,
              ),
              const CupertinoDivider(
                color: foregroundWhite,
              ),
              DisplayTrainLastInfo(
                trainData: trainData,
              ),
              const CupertinoDivider(),
              IntrinsicHeight(
                child: Row(
                  children: <Widget>[
                    // Expanded(
                    //   child: DisplayTrainNextStop(trainData: trainData,),
                    // ),
                    Expanded(
                      child: DisplayTrainRouteDuration(
                        trainData: trainData,
                      ),
                    ),
                    // Expanded(
                    //   child: DisplayTrainDestination(trainData: trainData,),
                    // ),
                    const SizedBox(
                      height: double.infinity,
                      child: CupertinoVerticalDivider(),
                    ),
                    Expanded(
                      child: DisplayTrainRouteDistance(
                        trainData: trainData,
                      ),
                    ),
                  ],
                ),
              ),
              const CupertinoDivider(
                color: foregroundWhite,
              ),
              if (onViewYesterdayTrain != null && trainData.stations.first.departure!.scheduleTime.compareTo(DateTime.now()) > 0) ...[
                DisplayTrainYesterdayWarningCupertino(onViewYesterdayTrain!),
                const CupertinoDivider(
                  color: foregroundWhite,
                ),
              ],
            ].map((e) => SliverToBoxAdapter(child: e)),
            // SliverToBoxAdapter(
            //   child: CupertinoDivider(),
            // ),
            // SliverToBoxAdapter(
            //   child: IntrinsicHeight(
            //     child: Row(
            //       children: <Widget>[
            //         // Expanded(
            //         //   child: DisplayTrainRouteDuration(trainData: trainData,),
            //         // ),
            //         Expanded(child: Container(),),
            //         SizedBox(
            //           height: double.infinity,
            //           child: CupertinoVerticalDivider(),
            //         ),
            //         Expanded(
            //           child: DisplayTrainRouteDistance(trainData: trainData,),
            //         )
            //       ],
            //     ),
            //   ),
            // ),
            DisplayTrainStations(
              trainData: trainData,
            ),
            SliverToBoxAdapter(
              child: Container(
                height: MediaQuery.of(context).viewPadding.bottom,
              ),
            ),
          ],
        );
      }),
    );
  }
}

class DisplayTrainID extends StatelessWidget {
  final TrainData trainData;
  const DisplayTrainID({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Center(
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text.rich(
        TextSpan(
          children: [
            TextSpan(
              text: trainData.rank,
              style: TextStyle(
                color: trainData.rank.startsWith('IR') ? const Color.fromARGB(255, 255, 0, 0) : null,
              ),
            ),
            const TextSpan(text: ' '),
            TextSpan(text: trainData.number,),
          ],
        ),
          style: CupertinoTheme.of(context).textTheme.navLargeTitleTextStyle,
        ),
      ),
    );
  }
}

class DisplayTrainRoute extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainRoute({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Row(
      children: <Widget>[
        Expanded(
          child: Center(
            child: Padding(
              padding: const EdgeInsets.all(4),
              child: Text(
                trainData.route.from,
                style:
                    CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                          fontSize: 16,
                        ),
              ),
            ),
          ),
        ),
        const Center(child: Text("-")),
        Expanded(
          child: Center(
            child: Padding(
              padding: const EdgeInsets.all(4),
              child: Text(
                trainData.route.to,
                style:
                    CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                          fontSize: 16,
                        ),
                textAlign: TextAlign.right,
              ),
            ),
          ),
        ),
      ],
    );
  }
}

class DisplayTrainOperator extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainOperator({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Center(
      child: Text(
        trainData.operator,
        style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
              fontSize: 14,
              fontStyle: FontStyle.italic,
            ),
      ),
    );
  }
}

class DisplayTrainDeparture extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainDeparture({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(2),
      child: Text(
        // "Plecare în ${dataPlecare.day.toString().padLeft(2, '0')}.${dataPlecare.month.toString().padLeft(2, '0')}.${dataPlecare.year.toString().padLeft(4, '0')}",
        "Plecare în ${trainData.date}",
        style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
              fontStyle: FontStyle.italic,
              fontWeight: FontWeight.w200,
            ),
        textAlign: TextAlign.center,
      ),
    );
  }
}

class DisplayTrainLastInfo extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainLastInfo({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    if (trainData.status == null) {
      return Container();
    }

    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Center(
          child: Padding(
            padding: const EdgeInsets.all(2),
            child: Text(
              "Ultima informație",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                    fontSize: 20,
                    fontWeight: FontWeight.bold,
                  ),
            ),
          ),
        ),
        Row(
          children: <Widget>[
            Padding(
              padding: const EdgeInsets.all(4),
              child: Text(
                trainData.status!.station,
                style: CupertinoTheme.of(context).textTheme.textStyle,
                textAlign: TextAlign.left,
              ),
            ),
            Expanded(
              child: Container(),
            ),
            Padding(
              padding: const EdgeInsets.all(4),
              child: Text(
                stateToString(trainData.status!.state),
                style: CupertinoTheme.of(context).textTheme.textStyle,
                textAlign: TextAlign.right,
              ),
            ),
          ],
        ),
        // FutureDisplay<DateTime>(
        //   future: trainData.lastInfo.dateAndTime,
        //   builder: (context, dt) {
        //     return Text(
        //       "Raportat în ${dt.day.toString().padLeft(2, '0')}.${dt.month.toString().padLeft(2, '0')}.${dt.year.toString().padLeft(4, '0')}, la ${dt.hour.toString().padLeft(2, '0')}:${dt.minute.toString().padLeft(2, '0')}",
        //       textAlign: TextAlign.center,
        //     );
        //   },
        // ),
        Builder(
          builder: (context) {
            final data = trainData.status!.delay;

            if (data == 0) {
              return Container();
            }

            if (data > 0) {
              return Text(
                "$data ${data == 1 ? 'minut' : 'minute'} întârziere",
                style:
                    CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                          fontSize: 14,
                          color: CupertinoColors.destructiveRed,
                        ),
              );
            } else {
              return Text(
                "${-data} ${data == -1 ? 'minut' : 'minute'} mai devreme",
                style:
                    CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                          fontSize: 12,
                          color: CupertinoColors.systemGreen,
                        ),
              );
            }
          },
        )
      ],
    );
  }
}

// class DisplayTrainNextStop extends StatelessWidget {
//   final TrainData trainData;
//
//   DisplayTrainNextStop({required this.trainData});
//
//   @override
//   Widget build(BuildContext context) {
//     return FutureBuilder(
//       future: trainData.nextStop.stationName,
//       builder: (context, snapshot) {
//         if (!snapshot.hasData) return Container();
//
//         return Column(
//           mainAxisSize: MainAxisSize.min,
//           children: <Widget>[
//             Padding(
//               padding: const EdgeInsets.all(4),
//               child: Text(
//                 "Următoarea oprire",
//                 style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
//                   fontSize: 20,
//                   fontWeight: FontWeight.bold,
//                 ),
//                 textAlign: TextAlign.center,
//               ),
//             ),
//             CupertinoDivider(
//               color: Color.fromRGBO(15, 15, 15, 1),
//             ),
//             FutureDisplay(
//               future: trainData.nextStop.stationName,
//               builder: (context, station) {
//                 return Padding(
//                   padding: const EdgeInsets.fromLTRB(4, 0, 4, 0),
//                   child: Text(
//                     station,
//                     style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
//                       fontSize: 18,
//                       fontWeight: FontWeight.w500,
//                     ),
//                     textAlign: TextAlign.center,
//                   ),
//                 );
//               },
//             ),
//             FutureDisplay<DateTime>(
//               future: trainData.nextStop.arrival,
//               builder: (context, arrival) {
//                 const months = ["ian", "feb", "mar", "apr", "mai", "iun", "iul", "aug", "sep", "oct", "noi", "dec"];
//
//                 return Column(
//                   mainAxisSize: MainAxisSize.min,
//                   children: <Widget>[
//                     Text(
//                       "în ${arrival.day} ${months[arrival.month - 1]} ${arrival.year}",
//                       style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
//                         fontSize: 14,
//                       ),
//                       textAlign: TextAlign.center,
//                     ),
//                     Text(
//                       "la ${arrival.hour.toString().padLeft(2, '0')}:${arrival.minute.toString().padLeft(2, '0')}",
//                       style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
//                         fontSize: 14,
//                       ),
//                       textAlign: TextAlign.center,
//                     ),
//                   ],
//                 );
//               },
//             )
//           ],
//         );
//       }
//     );
//   }
// }

class DisplayTrainDestination extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainDestination({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.all(4),
          child: Text(
            "Destinația",
            style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                  fontSize: 20,
                  fontWeight: FontWeight.bold,
                ),
            textAlign: TextAlign.center,
          ),
        ),
        const CupertinoDivider(
          color: Color.fromRGBO(15, 15, 15, 1),
        ),
        Padding(
          padding: const EdgeInsets.fromLTRB(4, 0, 4, 0),
          child: Text(
            trainData.stations.last.name,
            style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                  fontSize: 18,
                  fontWeight: FontWeight.w500,
                ),
            textAlign: TextAlign.center,
          ),
        ),
        Builder(
          builder: (context) {
            final arrival =
                trainData.stations.last.arrival!.scheduleTime.toLocal();
            final delay = trainData.stations.last.arrival!.status?.delay ?? 0;
            // final parts = arrival.split(':');
            // final arrivalDT = DateTime(DateTime.now().year, DateTime.now().month, DateTime.now().day, int.parse(parts[0]), int.parse(parts[1]));
            final arrivalWithDelay = arrival.add(Duration(minutes: delay));
            final arrivalWithDelayString =
                '${arrivalWithDelay.hour}:${arrivalWithDelay.minute.toString().padLeft(2, "0")}';
            // const months = ["ian", "feb", "mar", "apr", "mai", "iun", "iul", "aug", "sep", "oct", "noi", "dec"];

            return Column(
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                // Text(
                //   "în ${arrival.day} ${months[arrival.month - 1]} ${arrival.year}",
                //   style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                //     fontSize: 14,
                //   ),
                //   textAlign: TextAlign.center,
                // ),
                Text.rich(
                  TextSpan(
                    text: 'la',
                    children: [
                      const TextSpan(text: ' '),
                      TextSpan(
                        text:
                            '${arrival.hour.toString().padLeft(2, "0")}:${arrival.minute.toString().padLeft(2, "0")}',
                        style: delay == 0
                            ? null
                            : const TextStyle(
                                decoration: TextDecoration.lineThrough,
                              ),
                      ),
                      if (delay != 0) ...[
                        const TextSpan(text: ' '),
                        TextSpan(
                          text: arrivalWithDelayString,
                          style: TextStyle(
                            color: delay > 0
                                ? CupertinoColors.systemRed
                                : CupertinoColors.systemGreen,
                          ),
                        ),
                      ]
                    ],
                  ),
                  // "la ${arrival.hour.toString().padLeft(2, '0')}:${arrival.minute.toString().padLeft(2, '0')}",
                  style:
                      CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                            fontSize: 14,
                          ),
                  textAlign: TextAlign.center,
                ),
              ],
            );
          },
        )
      ],
    );
  }
}

class DisplayTrainRouteDistance extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainRouteDistance({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text(
          "Distanța rutei",
          style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                fontSize: 18,
                fontWeight: FontWeight.bold,
              ),
          textAlign: TextAlign.center,
        ),
        Text(
          "${trainData.stations.last.km} km",
          style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                fontSize: 16,
              ),
          textAlign: TextAlign.center,
        ),
      ],
    );
  }
}

class DisplayTrainRouteDuration extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainRouteDuration({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text(
          "Durata rutei",
          style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                fontSize: 18,
                fontWeight: FontWeight.bold,
              ),
          textAlign: TextAlign.center,
        ),
        Builder(
          builder: (context) {
            var duration =
                trainData.stations.last.arrival!.scheduleTime
                    .difference(
                      trainData.stations.first.departure!.scheduleTime);
            var durationString = StringBuffer();

            bool firstWritten = false;

            if (duration.inDays > 0) {
              firstWritten = true;
              if (duration.inDays == 1) {
                durationString.write("1 zi");
              } else {
                durationString.write("${duration.inDays} zile");
              }
              duration -= Duration(days: duration.inDays);
            }

            if (duration.inHours > 0) {
              if (firstWritten) {
                durationString.write(", ");
              }
              firstWritten = true;
              if (duration.inHours == 1) {
                durationString.write("1 oră");
              } else {
                durationString.write("${duration.inHours} ore");
              }
              duration -= Duration(hours: duration.inHours);
            }

            if (duration.inMinutes > 0) {
              if (firstWritten) {
                durationString.write(", ");
              }
              firstWritten = true;
              if (duration.inMinutes == 1) {
                durationString.write("1 minut");
              } else {
                durationString.write("${duration.inMinutes} minute");
              }
              duration -= Duration(minutes: duration.inMinutes);
            }

            return Text(
              durationString.toString(),
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                    fontSize: 16,
                  ),
              textAlign: TextAlign.center,
            );
          },
        ),
      ],
    );
  }
}

class DisplayTrainYesterdayWarningCupertino extends DisplayTrainYesterdayWarningCommon {
  const DisplayTrainYesterdayWarningCupertino(super.onViewYesterdayTrain, {super.key,});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Padding(
          padding: const EdgeInsets.all(8.0),
          child: Text.rich(
            TextSpan(
              children: [
                const TextSpan(text: DisplayTrainYesterdayWarningCommon.trainDidNotDepart,),
                const TextSpan(text: '\n'),
                TextSpan(
                  text: DisplayTrainYesterdayWarningCommon.seeYesterdayTrain,
                  style: const TextStyle(
                    color: CupertinoColors.link,
                  ),
                  recognizer: TapGestureRecognizer()
                    ..onTap = onViewYesterdayTrain,
                ),
              ],
            ),
            textAlign: TextAlign.center,
          ),
        ),
      ],
    );
  }
}

class DisplayTrainStations extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainStations({
    required this.trainData,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return SliverList(
      delegate: SliverChildBuilderDelegate(
        (context, index) {
          if (index.isOdd) {
            return const CupertinoDivider();
          } else {
            final itemIndex = index ~/ 2;
            return IndexedSemantics(
              index: itemIndex,
              child: DisplayTrainStation(
                station: trainData.stations[itemIndex],
              ),
            );
          }
        },
        childCount: trainData.stations.length * 2 - 1,
        addSemanticIndexes: false,
      ),
    );
  }
}
