import 'package:fluent_ui/fluent_ui.dart';
import 'package:flutter/gestures.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/pages/station_arrdep_page/view_station/view_station.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_fluent_DisplayTrainStation.dart';
import 'package:info_tren/utils/state_to_string.dart';

class TrainInfoLoadingFluent extends TrainInfoLoadingShared {
  TrainInfoLoadingFluent({required super.title, super.loadingText, super.key,});

  @override
  Widget build(BuildContext context) {
    return NavigationView(
      appBar: NavigationAppBar(
        title: Text(title),
      ),
      content: Center(
        child: loadingWidget,
      ),
    );
  }
}

class TrainInfoErrorFluent extends TrainInfoErrorShared {
  const TrainInfoErrorFluent({
    required super.error,
    required super.title,
    super.refresh,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return NavigationView(
      appBar: NavigationAppBar(
        title: Text(title),
      ),
      content: Center(
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Text(error.toString()),
            if (refresh != null)
              Padding(
                padding: const EdgeInsets.all(8),
                child: Button(
                  child: const Text('Retry'),
                  onPressed: () => refresh!(),
                ),
              ),
          ],
        ),
      ),
    );
  }
}

class TrainInfoFluent extends TrainInfoShared {
  const TrainInfoFluent({
    super.key,
    required super.trainData,
    super.isRefreshing,
    super.refresh,
    super.onViewYesterdayTrain,
  });

  @override
  Widget build(BuildContext context) {
    return Builder(
      builder: (context) {
        return NavigationView(
          appBar: NavigationAppBar(
            title: Text(
              "Informații despre ${trainData.rank} ${trainData.number}",
            ),
            actions: Row(
              mainAxisSize: MainAxisSize.min,
              children: [
                if (refresh != null) ...[
                  Center(
                    child: SizedBox(
                      height: 32,
                      width: 32,
                      child: IconButton(
                        icon: isRefreshing == true ? const ProgressRing() : const Icon(
                          FluentIcons.refresh,
                          size: 24,
                        ),
                        onPressed: isRefreshing == true ? null : () {
                          refresh!();
                        },
                      ),
                    ),
                  ),
                ],
              ],
            ),
          ),
          content: Column(
            children: <Widget>[
              Expanded(
                child: SafeArea(
                  bottom: false,
                  child: TrainInfoBody(
                    trainData: trainData,
                    onViewYesterdayTrain: onViewYesterdayTrain,
                  ),
                ),
              ),
            ],
          ),
        );
      },
    );
  }
}

class TrainInfoBodyFluent extends TrainInfoBodyShared {
  const TrainInfoBodyFluent({
    super.key,
    required super.trainData,
    super.onViewYesterdayTrain,
    super.refresh,
    super.isRefreshing,
  });

  @override
  Widget build(BuildContext context) {
    final mq = MediaQuery.of(context);

    if (mq.orientation == Orientation.landscape && mq.size.width >= 1000) {
      return Row(
        mainAxisSize: MainAxisSize.max,
        children: [
          Container(
            constraints: const BoxConstraints(
              minWidth: 400,
              maxWidth: 400,
            ),
            child: Column(
              mainAxisSize: MainAxisSize.max,
              children: [
                DisplayTrainID(trainData: trainData),
                DisplayTrainOperator(trainData: trainData),
                Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 2.0),
                  child: DisplayTrainRoute(trainData: trainData),
                ),
                DisplayTrainDeparture(trainData: trainData),
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: DisplayTrainLastInfo(trainData: trainData),
                ),
                IntrinsicHeight(
                  child: Row(
                    children: <Widget>[
                      Expanded(
                        child: Padding(
                          padding: const EdgeInsets.all(8.0),
                          child: DisplayTrainRouteDuration(
                            trainData: trainData,
                          ),
                        ),
                      ),
                      Expanded(
                        child: Padding(
                          padding: const EdgeInsets.all(8.0),
                          child: DisplayTrainRouteDistance(
                            trainData: trainData,
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
                const Divider(),
                if (onViewYesterdayTrain != null &&
                    trainData.stations.first.departure!.scheduleTime
                        .compareTo(DateTime.now()) >
                        0)
                  ...[
                    DisplayTrainYesterdayWarningFluent(
                      onViewYesterdayTrain!,
                    ),
                    const Divider(),
                  ],
              ],
            ),
          ),
          Expanded(
            child: CustomScrollView(
              slivers: [
                DisplayTrainStations(
                  trainData: trainData,
                ),
                SliverToBoxAdapter(
                  child: Container(
                    height: MediaQuery
                        .of(context)
                        .viewPadding
                        .bottom,
                  ),
                ),
              ],
            ),
          ),
        ],
      );
    }
    else {
      return CustomScrollView(
        slivers: <Widget>[
          SliverToBoxAdapter(
            child: DisplayTrainID(
              trainData: trainData,
            ),
          ),
          SliverToBoxAdapter(
            child: DisplayTrainOperator(
              trainData: trainData,
            ),
          ),
          SliverPadding(
            padding: const EdgeInsets.only(left: 2, right: 2),
            sliver: SliverToBoxAdapter(
              child: DisplayTrainRoute(
                trainData: trainData,
              ),
            ),
          ),
          SliverToBoxAdapter(
            child: DisplayTrainDeparture(
              trainData: trainData,
            ),
          ),
          SliverToBoxAdapter(
            child: DisplayTrainLastInfo(
              trainData: trainData,
            ),
          ),
          SliverToBoxAdapter(
            child: IntrinsicHeight(
              child: Row(
                children: <Widget>[
                  Expanded(
                    child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: DisplayTrainRouteDuration(
                        trainData: trainData,
                      ),
                    ),
                  ),
                  Expanded(
                    child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: DisplayTrainRouteDistance(
                        trainData: trainData,
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
          const SliverToBoxAdapter(
            child: Divider(),
          ),
          if (onViewYesterdayTrain != null &&
              trainData.stations.first.departure!.scheduleTime
                  .compareTo(DateTime.now()) >
                  0) ...[
            SliverToBoxAdapter(
              child: DisplayTrainYesterdayWarningFluent(
                  onViewYesterdayTrain!),
            ),
            const SliverToBoxAdapter(
              child: Divider(),
            ),
          ],
          DisplayTrainStations(
            trainData: trainData,
          ),
          SliverToBoxAdapter(
            child: Container(
              height: MediaQuery
                  .of(context)
                  .viewPadding
                  .bottom,
            ),
          ),
        ],
      );
    }
  }
}

class DisplayTrainID extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainID({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Text.rich(
      TextSpan(
        children: [
          TextSpan(
            text: trainData.rank,
            style: TextStyle(
              color: trainData.rank.startsWith('IR')
                  ? const Color.fromARGB(255, 255, 0, 0)
                  : null,
            ),
          ),
          const TextSpan(text: ' '),
          TextSpan(
            text: trainData.number,
          ),
        ],
      ),
      style: FluentTheme.of(context).typography.title?.copyWith(
        color: FluentTheme.of(context).typography.body?.color,
        fontWeight: FontWeight.bold,
      ),
      textAlign: TextAlign.center,
    );
  }
}

class DisplayTrainOperator extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainOperator({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Text(
      trainData.operator,
      style: FluentTheme.of(context).typography.body?.copyWith(
        fontStyle: FontStyle.italic,
        fontSize: 14,
      ),
      textAlign: TextAlign.center,
    );
  }
}

class DisplayTrainRoute extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainRoute({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Row(
      children: <Widget>[
        Expanded(
          child: Center(
            child: Padding(
              padding: const EdgeInsets.all(4),
              child: Text(
                trainData.route.from,
                style: FluentTheme.of(context).typography.body?.copyWith(
                  fontSize: 16,
                ),
              ),
            ),
          ),
        ),
        const Center(child: Text("-")),
        Expanded(
          child: Center(
            child: Padding(
              padding: const EdgeInsets.all(4),
              child: Text(
                trainData.route.to,
                style: FluentTheme.of(context).typography.body?.copyWith(
                  fontSize: 16,
                ),
                textAlign: TextAlign.right,
              ),
            ),
          ),
        ),
      ],
    );
  }
}

class DisplayTrainDeparture extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainDeparture({required this.trainData, super.key});

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(2),
      child: Text(
        // "Plecare în ${dataPlecare.day.toString().padLeft(2, '0')}.${dataPlecare.month.toString().padLeft(2, '0')}.${dataPlecare.year.toString().padLeft(4, '0')}",
        "Plecare în ${trainData.date}",
        style: FluentTheme.of(context).typography.body?.copyWith(
          fontStyle: FontStyle.italic,
          fontWeight: FontWeight.w200,
          fontSize: 16,
        ),
        textAlign: TextAlign.center,
      ),
    );
  }
}

class DisplayTrainLastInfo extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainLastInfo({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    if (trainData.status == null) {
      return Container();
    }

    return Card(
      child: Padding(
        padding: const EdgeInsets.all(2),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Center(
              child: Padding(
                padding: const EdgeInsets.all(2),
                child: Text(
                  "Ultima informație",
                  style: FluentTheme.of(context).typography.body?.copyWith(
                    fontSize: 22,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
            ),
            Row(
              children: <Widget>[
                Padding(
                  padding: const EdgeInsets.all(4),
                  child: Text(
                    trainData.status!.station,
                    style: FluentTheme.of(context).typography.body?.copyWith(
                      fontSize: 18,
                    ),
                    textAlign: TextAlign.left,
                  ),
                ),
                Expanded(
                  child: Container(),
                ),
                Padding(
                  padding: const EdgeInsets.all(4),
                  child: Text(
                    stateToString(trainData.status!.state),
                    style: FluentTheme.of(context).typography.body?.copyWith(
                      fontSize: 18,
                    ),
                    textAlign: TextAlign.right,
                  ),
                ),
              ],
            ),
            Padding(
              padding: const EdgeInsets.all(2),
              child: Row(
                children: <Widget>[
                  Expanded(
                    child: Container(),
                  ),
                  Builder(
                    builder: (context) {
                      final data = trainData.status!.delay;
                      if (data == 0) {
                        return Container();
                      }

                      if (data > 0) {
                        return Text(
                          "$data ${data == 1 ? 'minut' : 'minute'} întârziere",
                          style:
                          FluentTheme.of(context).typography.body?.copyWith(
                            fontSize: 16,
                            color: Colors.red.lighter,
                            // color: Colors.red.shade300,
                          ),
                        );
                      } else {
                        return Text(
                          "${-data} ${data == -1 ? 'minut' : 'minute'} mai devreme",
                          style:
                          FluentTheme.of(context).typography.body?.copyWith(
                            fontSize: 16,
                            color: Colors.green.lighter,
                            // color: Colors.green.shade300,
                          ),
                        );
                      }
                    },
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class DisplayTrainDestination extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainDestination({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    final destination = trainData.stations.last;

    return Card(
      child: Center(
        child: Padding(
          padding: const EdgeInsets.all(2),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Padding(
                padding: const EdgeInsets.all(4),
                child: Text(
                  "Destinația",
                  style: FluentTheme.of(context).typography.body?.copyWith(
                    fontSize: 22,
                    fontWeight: FontWeight.bold,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              Padding(
                padding: const EdgeInsets.fromLTRB(4, 0, 4, 0),
                child: Text(
                  destination.name,
                  style: FluentTheme.of(context).typography.body?.copyWith(
                    fontSize: 20,
                    fontWeight: FontWeight.w500,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              Builder(
                builder: (context) {
                  final arrival = destination.arrival!.scheduleTime.toLocal();
                  final delay =
                      trainData.stations.last.arrival!.status?.delay ?? 0;
                  final arrivalWithDelay =
                  arrival.add(Duration(minutes: delay));
                  final arrivalWithDelayString =
                      '${arrivalWithDelay.hour}:${arrivalWithDelay.minute.toString().padLeft(2, "0")}';
                  // const months = ["ian", "feb", "mar", "apr", "mai", "iun", "iul", "aug", "sep", "oct", "noi", "dec"];

                  return Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      // Text(
                      //   "în ${arrival.day} ${months[arrival.month - 1]} ${arrival.year}",
                      //   style: Theme.of(context).textTheme.bodyText2?.copyWith(
                      //     fontSize: isSmallScreen(context) ? 12 : 14,
                      //   ),
                      //   textAlign: TextAlign.center,
                      // ),
                      Text.rich(
                        TextSpan(
                          text: 'la',
                          children: [
                            const TextSpan(text: ' '),
                            TextSpan(
                              text:
                              '${arrival.hour.toString().padLeft(2, '0')}:${arrival.minute.toString().padLeft(2, '0')}',
                              style: delay == 0
                                  ? null
                                  : const TextStyle(
                                decoration: TextDecoration.lineThrough,
                              ),
                            ),
                            if (delay != 0) ...[
                              const TextSpan(text: ' '),
                              TextSpan(
                                text: arrivalWithDelayString,
                                style: TextStyle(
                                  color: delay > 0
                                      ? Colors.red.lighter
                                      : Colors.green.lighter,
                                  // color: delay > 0
                                  //     ? Colors.red.shade300
                                  //     : Colors.green.shade300,
                                ),
                              ),
                            ]
                          ],
                        ),
                        style: FluentTheme.of(context).typography.body?.copyWith(
                          fontSize: 16,
                        ),
                        textAlign: TextAlign.center,
                      ),
                    ],
                  );
                },
              )
            ],
          ),
        ),
      ),
    );
  }
}

class DisplayTrainRouteDistance extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainRouteDistance({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Card(
      child: Center(
        child: Padding(
          padding: const EdgeInsets.all(2),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Text(
                "Distanța rutei",
                style: FluentTheme.of(context).typography.body?.copyWith(
                  fontSize: 22,
                  fontWeight: FontWeight.bold,
                ),
                textAlign: TextAlign.center,
              ),
              Text(
                "${trainData.stations.last.km} km",
                style: FluentTheme.of(context).typography.body?.copyWith(
                  fontSize: 20,
                ),
                textAlign: TextAlign.center,
              ),
            ],
          ),
        ),
      ),
    );
  }
}

class DisplayTrainRouteDuration extends StatelessWidget {
  final TrainData trainData;

  const DisplayTrainRouteDuration({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return Card(
      child: Center(
        child: Padding(
          padding: const EdgeInsets.all(2),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Text(
                "Durata rutei",
                style: FluentTheme.of(context).typography.body?.copyWith(
                  fontSize: 22,
                  fontWeight: FontWeight.bold,
                ),
                textAlign: TextAlign.center,
              ),
              Builder(
                builder: (context) {
                  var duration = trainData.stations.last.arrival!.scheduleTime
                      .difference(
                      trainData.stations.first.departure!.scheduleTime);
                  var durationString = StringBuffer();

                  bool firstWritten = false;

                  if (duration.inDays > 0) {
                    firstWritten = true;
                    if (duration.inDays == 1) {
                      durationString.write("1 zi");
                    } else {
                      durationString.write("${duration.inDays} zile");
                    }
                    duration -= Duration(days: duration.inDays);
                  }

                  if (duration.inHours > 0) {
                    if (firstWritten) {
                      durationString.write(", ");
                    }
                    firstWritten = true;
                    if (duration.inHours == 1) {
                      durationString.write("1 oră");
                    } else {
                      durationString.write("${duration.inHours} ore");
                    }
                    duration -= Duration(hours: duration.inHours);
                  }

                  if (duration.inMinutes > 0) {
                    if (firstWritten) {
                      durationString.write(", ");
                    }
                    firstWritten = true;
                    if (duration.inMinutes == 1) {
                      durationString.write("1 minut");
                    } else {
                      durationString.write("${duration.inMinutes} minute");
                    }
                    duration -= Duration(minutes: duration.inMinutes);
                  }

                  return Text(
                    durationString.toString(),
                    style: FluentTheme.of(context).typography.body?.copyWith(
                      fontSize: 20,
                    ),
                    textAlign: TextAlign.center,
                  );
                },
              ),
            ],
          ),
        ),
      ),
    );
  }
}

class DisplayTrainYesterdayWarningFluent
    extends DisplayTrainYesterdayWarningCommon {
  const DisplayTrainYesterdayWarningFluent(super.onViewYesterdayTrain, {super.key,});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Padding(
          padding: const EdgeInsets.all(8.0),
          child: Text.rich(
            TextSpan(
              children: [
                const TextSpan(
                  text: DisplayTrainYesterdayWarningCommon.trainDidNotDepart,
                ),
                const TextSpan(text: '\n'),
                TextSpan(
                  text: DisplayTrainYesterdayWarningCommon.seeYesterdayTrain,
                  style: TextStyle(
                    color: Colors.blue,
                  ),
                  recognizer: TapGestureRecognizer()
                    ..onTap = onViewYesterdayTrain,
                ),
              ],
            ),
            textAlign: TextAlign.center,
          ),
        ),
      ],
    );
  }
}

class DisplayTrainStations extends StatelessWidget {
  final TrainData trainData;
  const DisplayTrainStations({required this.trainData, super.key,});

  @override
  Widget build(BuildContext context) {
    return SliverList(
      delegate: SliverChildBuilderDelegate(
            (context, index) {
          return IndexedSemantics(
            index: index,
            child: DisplayTrainStation(
              station: trainData.stations[index],
              onTap: () {
                Navigator.of(context).pushNamed(
                  ViewStationPage.routeName,
                  arguments: ViewStationArguments(stationName: trainData.stations[index].name),
                );
              },
            ),
          );
        },
        childCount: trainData.stations.length,
        addSemanticIndexes: true,
      ),
    );
  }
}
